setwd("~/Works in progress/Papers and projects/Little green book")
library(rem)
library(data.table)
load('Marijuana relational event data.RData')


marijuana$Time[which(marijuana$Time==1.2017)]<-1
marijuana$Time[which(marijuana$Time==2.2017)]<-1
marijuana$Time[which(marijuana$Time==3.2017)]<-2
marijuana$Time[which(marijuana$Time==4.2017)]<-3
marijuana$Time[which(marijuana$Time==5.2017)]<-4
marijuana$Time[which(marijuana$Time==6.2017)]<-5
marijuana$Time[which(marijuana$Time==7.2017)]<-5 ##market shut down in July 2017 for a month
marijuana$Time[which(marijuana$Time==8.2017)]<-6
marijuana$Time[which(marijuana$Time==9.2017)]<-7
marijuana$Time[which(marijuana$Time==10.2017)]<-8
marijuana$Time[which(marijuana$Time==11.2017)]<-9
marijuana$Time[which(marijuana$Time==12.2017)]<-10
marijuana$Time[which(marijuana$Time==1.2018)]<-11
marijuana$Time[which(marijuana$Time==2.2018)]<-11 ###only 3 or so transactions in February


##create lagged reputation measure
marijuana<-as.data.table(marijuana)
marijuana<-marijuana[, Lag.rep:=c(NA, Reputation[-.N]), by=Seller]

###replace missing with 0
marijuana$Lag.rep[which(is.na(marijuana$Lag.rep))]<-0

###sort data by time for processing
marijuana$Time<-as.numeric(marijuana$Time)
marijuana<-marijuana[order(marijuana$Time),]

###now create REM dataset--this might take a moment
REM.data<-createRemDataset(
  data = marijuana, sender = marijuana$Buyer,
  target = marijuana$Seller, eventSequence = marijuana$Time)

###merge in vendor attributes
marijuana$eventTime<-marijuana$Time
marijuana$target<-marijuana$Seller
REM.data<-merge(REM.data,unique(marijuana[,c("target","Price")]))
REM.data<-merge(REM.data,unique(marijuana[,c("target","eventTime","Lag.rep")]))




###now we can calculate network statistics
REM.data<-REM.data[order(REM.data$eventTime),]

#four-cycle closure
REM.data$four.cycles<-fourCycleStat(time = REM.data$eventTime,sender = REM.data$sender,eventvar = REM.data$eventDummy,
                                    target=REM.data$target,halflife = .7,inParallel = TRUE,cluster = 4)
#inertia is the tendency for i to repeat transactions with j
REM.data$inertia<-inertiaStat(time = REM.data$eventTime,sender = REM.data$sender,eventvar = REM.data$eventDummy,
                              target=REM.data$target,halflife = .7,inParallel = TRUE,cluster = 4)
#buyer/vendor degree centrality
REM.data$vend.deg<-degreeStat(time = REM.data$eventTime,degreevar = REM.data$target,eventvar = REM.data$eventDummy,
                              halflife = .7,inParallel = TRUE,cluster = 4)
REM.data$buy.deg<-degreeStat(time = REM.data$eventTime,degreevar = REM.data$sender,eventvar = REM.data$eventDummy,
                             halflife = .7,inParallel = TRUE,cluster = 4)





###estimate the model

library(survival)
model<-clogit(eventDummy~Lag.rep+
                 Price+
                 four.cycles+
                 inertia+
                 vend.deg+
                 buy.deg,
               data=REM.data,
               method="approximate")

AIC(model)
BIC(model)

###marginal effects
beta<-coef(model)
p<-exp(model$linear.predictors)/(1+exp(model$linear.predictors))

vars<-names(coef(model))

##this function calculates the average marginal effect for a single variable
AME.fun<-function(theta){
  
  ME.REM<-sapply(names(theta),function(x)
    (p*(1-p)*theta[var1]))
  mean(ME.REM,na.rm = TRUE)}

AME_out<-matrix(NA,nrow=length(vars),ncol=4)  #create a matrix to store output

colnames(AME_out)<-c("AME","SE","Z","p")
rownames(AME_out)<-vars

##Now we loop over all variables
for(i in 1:length(vars)){ 
  
  theta<-beta[i]
  var1<-vars[i]
  
  AME_out[i,1]<-AME<-AME.fun(theta)
  
  ##variance estimates
  Jac<-numDeriv::jacobian(AME.fun,theta) ##delta standard errors
  vc<-vcov(model)[var1,var1]
  variance.ame<-Jac%*%vc%*%t(Jac)
  
  AME_out[i,2]<-AME.se<-sqrt(variance.ame)
  AME_out[i,3]<-AME/AME.se
  AME_out[i,4]<-2*pnorm(q=AME/AME.se, lower.tail=FALSE)
  
  
}


##average marginal effects
AME_out



###let's allow for heterogeneity among buyers
clust.model<-clogit(eventDummy~Lag.rep+
                Price+
                four.cycles+
                inertia+
                vend.deg+
                buy.deg+
                  cluster(sender),
              data=REM.data,
              method="approximate")

clust.model
AIC(clust.model)
BIC(clust.model)

###now we allow for vendor fixed effects
strat.model<-clogit(eventDummy~Lag.rep+
                Price+
                four.cycles+
                inertia+
                vend.deg+
                buy.deg+
                cluster(sender)+
                strata(target),
              data=REM.data,
              method="approximate")

strat.model
AIC(strat.model)
BIC(strat.model)


###marginal effects
beta<-coef(strat.model)
p<-exp(strat.model$linear.predictors)/(1+exp(strat.model$linear.predictors))

vars<-names(coef(strat.model)) # 

##this function calculates the average marginal effect for a single variable
AME.fun<-function(theta){
  
  ME.REM<-sapply(names(theta),function(x)
    (p*(1-p)*theta[var1]))
  mean(ME.REM,na.rm = TRUE)}

AME_out<-matrix(NA,nrow=length(vars),ncol=4) #create a matrix to store output

colnames(AME_out)<-c("AME","SE","Z","p")
rownames(AME_out)<-vars

for(i in 1:length(vars)){ #loop over all variables
  
  theta<-beta[i]
  var1<-vars[i]
  
  AME_out[i,1]<-AME<-AME.fun(theta)
  
  ##variance estimates
  Jac<-numDeriv::jacobian(AME.fun,theta) ##delta standard errors
  vc<-vcov(strat.model)[var1,var1]
  variance.ame<-Jac%*%vc%*%t(Jac)
  
  AME_out[i,2]<-AME.se<-sqrt(variance.ame)
  AME_out[i,3]<-AME/AME.se
  AME_out[i,4]<-2*pnorm(q=AME/AME.se, lower.tail=FALSE)
  
  
}

AME_out




###account for time trends
time.model<-clogit(eventDummy~Lag.rep+
                Price+
                four.cycles+
                inertia+
                vend.deg+
                buy.deg+
                cluster(sender)+
                strata(target)+
               eventTime+
                 I(eventTime^2)+
                 I(eventTime^3),
              data=REM.data,
              method="approximate")


time.model
AIC(time.model)
BIC(time.model)






