######################
#   TNAM
#####################


###The tnam package is archived, so we have to do a manual install
url<-"https://cran.r-project.org/src/contrib/Archive/tnam/"
pkgFile<-"tnam_1.6.5.tar.gz"
download.file(url=url,destfile=pkgFile)

setwd("C:/Users/sduxb/Downloads") ##change directory to where tar.gz file is downloaded
install.packages(pkgs=pkgFile,type="source",repos=NULL) 


###now we can load tnam
library(tnam)
library(RSiena) ##we'll load RSiena since we're using data from here.
###create a list of adjacency matrices
network_array<-list(s501,s502,s503)


smoking<-as.data.frame(s50s) ##load smoking data--TNAM assumes a dataframe
##for our analysis, we'll look at binary smoking behavior
for(i in 1:ncol(smoking)){
  
  smoking[,i][smoking[,i]>1]<-2
}

alcohol<-as.data.frame(s50a) ##we'll use alcohol consumption as a covariate as well

###now we look at tnam terms
?`tnam-terms`


##let's start by calculating the offset matrix

weight_mat_dist1<-netlag(y=smoking,
                   networks=network_array,
                   pathdist=1)

##create our primary dataframe
main_mat<-weight_mat_dist1

##now let's add in exogenous attributes
main_mat$alcohol<-c(alcohol$V1,alcohol$V2,alcohol$V3)



##and calculate some endogenous attributes of interest
main_mat$idegree<-centrality(network_array,type="indegree")[,1] ##indegree
main_mat$odegree<-centrality(network_array,type="outdegree")[,1] ##outdegree
main_mat$bdegree<-centrality(network_array,type="betweenness")[,1] ##outdegree
main_mat$clustering<-clustering(network_array,type="clustering")[,1] ##local clustering coefficient



##test for autocorrelation
for(i in 1:length(network_array)){
  
  print(ape::Moran.I(network_array[[i]],x=smoking[,i]))
}



##prepare data for estimation
main_mat$smoking<-main_mat$response-1


###Now let's estimate the model
TNAM_1<-glm(smoking~netlag.pathdist1+
              alcohol+
              idegree+
              odegree+
              bdegree+
              clustering,
            data=main_mat,
            family="binomial") ##use a logistic link function
AIC(TNAM_1)
BIC(TNAM_1)




###include for node-level random effect
library(lme4)
TNAM_RE<-glmer(smoking~netlag.pathdist1+
               alcohol+
               idegree+
               odegree+
               bdegree+
               clustering+
               (1|node),
             data=main_mat,
             family="binomial") ##use a logistic link function
AIC(TNAM_RE)
BIC(TNAM_RE)


###control for node fixed effects
TNAM_FE<-glm(smoking~netlag.pathdist1+
              alcohol+
              idegree+
              odegree+
              bdegree+
              clustering+
              as.factor(node),
            data=main_mat,
            family="binomial") ##use a logistic link function
AIC(TNAM_FE)
BIC(TNAM_FE)



###control for time
TNAM_RE2<-glmer(smoking~netlag.pathdist1+
               alcohol+
               idegree+
               odegree+
               bdegree+
               clustering+
                time+
                 (1|node),
             data=main_mat,
             family="binomial") ##use a logistic link function
AIC(TNAM_RE2)
BIC(TNAM_RE2)



##stepwise selection of pathdistance
main_mat$netlag.pathdist2<-netlag(y=smoking,
                                  networks=network_array,
                                  pathdist=2)[,1]
main_mat$netlag.pathdist3<-netlag(y=smoking,
                                  networks=network_array,
                                  pathdist=3)[,1]

path2_mod<-glm(smoking~netlag.pathdist1+
                 netlag.pathdist2+
                 alcohol+
                 idegree+
                 odegree+
                 bdegree+
                 clustering,
               data=main_mat,
               family="binomial") 

path3_mod<-glm(smoking~netlag.pathdist1+
                 netlag.pathdist2+
                 netlag.pathdist3+
                 alcohol+
                 idegree+
                 odegree+
                 bdegree+
                 clustering,
               data=main_mat,
               family="binomial") 

##compare fit
AIC(TNAM_1)
AIC(path2_mod)
AIC(path3_mod)

##compare fit
BIC(TNAM_1)
BIC(path2_mod)
BIC(path3_mod)

















########################################
#       Co-evolution model
#######################################
library(RSiena)



##create "sienaDependent" object,
TLSnet<-sienaDependent(array(c(network_array[[1]],
                                       network_array[[2]],
                                       network_array[[3]]),
                                       dim=c(50,50,3))) ##each network is 50 x 50, observed 3 times


###now we have to do the same thing, but for the behavior
TLSbeh<-sienaDependent(as.matrix(smoking),type="behavior") 


##set covariates
Alcohol<-varCovar(as.matrix(alcohol))


###create dataset, but specify network AND behavior
SAOM.Data<-sienaDataCreate(Network=TLSnet,
                           Behavior=TLSbeh,
                           Alcohol)

###Create the effects object
SAOM.terms<-getEffects(SAOM.Data)     #Right now we only have the rate parameters, density, and reciprocity


##look up available terms
effectsDocumentation(SAOM.terms)

###We'll start by specifying the NETWORK function

##include ego, alter, and alcohol homophily for alcohol in the network function
SAOM.terms<-includeEffects(SAOM.terms,egoX,altX,absDiffX,interaction1="Alcohol") #this includes a sender effect (egoX), receiver effect (alterX), and homophily effect (sameX) for alcohol

#Do the same for smoking behavior in the network function
  ##Note that the "behavior" label is generic for the dependent behavioral variable
SAOM.terms<-includeEffects(SAOM.terms,egoX,altX,sameX,interaction1="Behavior") #this includes a sender effect (egoX), receiver effect (alterX), and homophily effect (sameX) for smoking


##now include endogenous processes into network function
SAOM.terms<-includeEffects(SAOM.terms,transTies,inPop)



###Now let's specify the BEHAVIOR function
  
##Include alcohol consumption as predictor of smoking behavior
SAOM.terms<-includeEffects(SAOM.terms,effFrom,name="Behavior",interaction1="Alcohol")

###Now let's include endogenous processes
SAOM.terms<-includeEffects(SAOM.terms,totSim,name="Behavior",interaction1="Network") ##includes the total smoking behavior of ego's alters

SAOM.terms<-includeEffects(SAOM.terms,isolate,
                           name="Behavior",interaction1="Network") ##includes isolate status as predictors of smoking behavior





###Now we have to create and estimate the model

###Now that all of our effects are set, we create the model
create.model<-sienaAlgorithmCreate(projname="Chapter5", ##the output on model fit and convergence is stored in a "Chapter5.txt" file
                                   seed=21093, ###ensures reproducible results
                                   nsub=4,   
                                   n3=1000) 


##now we can estimate the model using siena07
TLSmodel<-siena07(create.model,
                        data=SAOM.Data,
                        effects=SAOM.terms,
                        verbose=T,  ##verbose allows us to keep an eye on estimation
                        returnDeps=T) #this returns the simulated networks used to assess convergence--we use them in the sienaGOF function to evaluate fit

TLSmodel




###check GOF
indegGOF<-sienaGOF(TLSmodel,IndegreeDistribution,join = T,varName="Network")
outdegGOF<-sienaGOF(TLSmodel,OutdegreeDistribution,join = T,varName="Network")
TriadGOF<-sienaGOF(TLSmodel,TriadCensus,join = T,varName="Network")
behGOF<-sienaGOF(TLSmodel,BehaviorDistribution,join = T,varName="Behavior")



pdf("RSIENA_coevolve_GOF.pdf")
plot(indegGOF)
plot(outdegGOF)
plot(TriadGOF)
plot(behGOF)

dev.off()


##score test of insignificant behavior parameters
Multipar.RSiena(TLSmodel,17,18) #not jointly significant




##Time heterogenetiy tests--no time heterogenetiy
timetests<-sienaTimeTest(TLSmodel)




####How much selection versus how much influence?
selectionTable(TLSmodel,SAOM.Data,"Network","Behavior",levls=0:1)
influenceTable(TLSmodel,SAOM.Data,"Network","Behavior",levls=0:1)




library(ggplot2)

ggdata<-data.frame(value=c(-.2878,-.2236,-.4344,.1134,.2222,-.4412,-.5000,.2810),
                   ego=c(0,0,1,1,0,1,0,1),
                   Alter=c(0,1,0,1,0,0,1,1),
                   Type=c(rep("Network",4),rep("Smoking Behavior",4)))

ggdata$Alter<-as.factor(ggdata$Alter)
levels(ggdata$Alter)<-c("Non-smoker","Smoker")
ggdata$ego<-as.factor(ggdata$ego)
levels(ggdata$ego)<-c("Non-smoker","Smoker")


ggplot(ggdata,aes(x=ego,y=exp(value)))+geom_point()+geom_line(aes(linetype=Alter,group=Alter))+
  facet_grid(.~Type)+theme_bw()+xlab("Ego smoking behavior")+ylab("Odds ratio")

